unit config;

(*
  This is a configuration file wrapper
  Based on TminiIni - ie a fancy ini file

  What this provides is support for a hierarchical key naming scheme
  (imagine a simpler form of registry in the file of your choice)

  Imagine an ini file that looks like this:-
  ---------------------------------------------------------------------
    user.1.name=James T Kirk
    user.1.password=wecomeinpeace
    user.2.name=Mr Spock
    user.2.password=illogical
    server.mailin=  ; incoming mail servers
    server.mailin.1.ip=123.123.123.123
    server.mailin.1.userID=enterprise
    server.mailin.1.password=stardate

  Then we can use it as follows...
  ---------------------------------------------------------------------
  .countkey('server.mailout') --> 0    "does key exist?"

  .countkey('user') --> 4              "raw count of how many lines start with this key"

  .countSubKeys('user')-->2            "No of distinct sub-keys"

  .listSubKeys('user') --> '1|2'       string of keys split by vertical bar

  .subTree('user') --> string list of everything starting with 'user' with user key stripped off
                     eg .subTree('user.1') --> string list
                                               [line 1]:- name=James T Kirk
                                               [line 2]:- password=wecomeinpeace

  .AddTree - opposite of SubTree - puts string list into 'tree'

  .get... and set... work as TminiIni but remember to use the full key along the lines of
                     .setstr('user.'+userid+'.password','easycrack')

  .remove() removes all lines starting with this key eg delete('user.1')


  * The text version on file will always be sorted
  * Comments (;...) on the end of lines are preserved except when being deleted
  * This only works with strings - You'll have to convert other things yourself

*)


interface

uses     miniini,Classes,safestring,sysutils,forms,MessageDialogBx;

type
   TConfig = class(TMiniIni)
     private
       function getsubkey(const partkey,fullkey:string):string;
     public
       constructor create(const aFilename:string);  override;
       procedure   addTree(const ky:string;  sl:tstringlist);
       function    countkey(const akey:string):integer;
       function    countSubkeys(const akey:string):integer;
       function    getstr(const ky:string):string;                   override;
       procedure   remove(const aKey:string);
       function    listSubKeys(const akey:string):string;
       function    SubTree(const akey:string):Tstringlist;
       procedure   setstr(const ky,v:string);                        override;
       procedure   setInt(const ky:string;  const v:integer);        override;
       procedure   setDateTime(const ky:string; const v:TDatetime);  override;
       procedure   setBool(const ky:string;  const v:boolean);       override;
   end;


implementation


constructor Tconfig.create(const aFilename:string);
{ load as for TminiIni but make sure it is sorted (v-important) }
begin
  inherited create(aFilename);
  sort;
end;


function TConfig.countkey(const akey:string):integer;
{ how many lines start with this key? }
var
  i : integer;
begin
  result := 0;
  for i := 0 to count-1 do begin
    if pos(akey,names[i])=1 then result := result + 1;
  end;
end;


function TConfig.countSubKeys(const akey:string):integer;
{ how many DIFFERENT sub keys are there? }
var
  i : integer;
  s,sk,n : string;
begin
  result := 0;
  for i := 0 to count-1 do begin
    n := names[i];
    if pos(akey,n)=1 then begin
      s := getsubkey(akey,n);
      if s <> sk then begin
        result := result + 1;
        sk := s;
      end;
    end;
  end;
end;


function TConfig.listSubKeys(const akey:string):string;
{ return a barred string of the DIFFERENT sub keys }
var
  i : integer;
  s,sk,n : string;
begin
  result := '';
  for i := 0 to count-1 do begin
    n := names[i];
    if pos(akey,n)=1 then begin
      s := getsubkey(akey,n);
      if s <> sk then begin
        result := result + '|'+s;
        sk := s;
      end;
    end;
  end;
  if result <> '' then result := copy(result,2,999);
end;

function TConfig.Subtree(const akey:string):tstringlist;
{ return a stringlist of ALL the items with this key stem
  only partial keys are returned
}
var
  i : integer;
  s : string;
begin
  result := tstringlist.create;
  for i := 0 to count-1 do begin
    s := strings[i];
    if pos(akey,s)=1 then begin
      s := copy(s,length(akey)+2,999);
      result.add(s);
    end;
  end;
end;

procedure   TConfig.addTree(const ky:string;  sl:tstringlist);
{ add a complete set of items as a sub-tree
  complimentary function to SubTree

  format of strings should be partial key=value
}
var
  i : integer;
  n,s : string;
begin
  for i := 0 to sl.count-1 do begin
    n := ky+'.'+sl.names[i];
    remove(n);
    s := ky+'.'+sl.strings[i];
    add(s);
  end;
  sort;
end;





procedure Tconfig.remove(const aKey:string);
{ delete all items starting with this key }
var
  i : integer;
begin
  for i := count-1 downto 0 do begin
    if pos(aKey,names[i])=1 then begin
      inherited delete(i);
      Changed := true;
    end;
  end;
end;

function   TConfig.getstr(const ky:string):string;
{NB strings are safe against control chars }
begin
  result := inherited getstr(ky);
  result := readsafestring(result);
  sort;
end;


procedure   TConfig.setstr(const ky,v:string);
{NB strings are safe against control chars }
begin
  inherited setstr(ky,MakeSafeString(v));
  sort;
end;

procedure   TConfig.setInt(const ky:string;  const v:integer);
begin
  inherited setInt(ky,v);
  sort;
end;

procedure   TConfig.setDateTime(const ky:string; const v:TDatetime);
begin
  inherited setDateTime(ky,v);
  sort;
end;

procedure   TConfig.setBool(const ky:string;  const v:boolean);
begin
  inherited setBool(ky,v);
  sort;
end;





function TConfig.getsubkey(const partkey,fullkey:string):string;
{ get the next part of the key
  eg  getsubkey('foo.bar','foo.bar.fox.box')-->'fox'}
var
  i : integer;
  s : string;
begin
  result := '';
  if pos(partkey,fullkey)=1 then begin  // trap silly
    i := length(partkey);
    if i < length(fullkey) then begin   // trap equal
      s := copy(fullkey,i+2,999);        // s is now remainder of key following dot splitter
      i := pos('.',s);
      if i > 0 then s := copy(s,1,i-1); // lose any grandchild keys
      result := s;
    end;
  end;
end;



end.
